import { Injectable } from '@angular/core';
import {
  ActivatedRouteSnapshot,
  Resolve,
  Router,
  RouterStateSnapshot,
} from '@angular/router';
import { catchError, Observable, throwError } from 'rxjs';

import { PartnerBranch } from '../models/partner-branch';
import { PartnerBranchPagination } from '../models/partner-branch-pagination';
import { PartnerBranchService } from '../services/partner-branch.service';

@Injectable({
  providedIn: 'root',
})
export class PartnerBranchResolver implements Resolve<any> {
  /**
   * Constructor
   */
  constructor(
    private _partnerBranchService: PartnerBranchService,
    private _router: Router
  ) {}

  // -----------------------------------------------------------------------------------------------------
  // @ Public methods
  // -----------------------------------------------------------------------------------------------------

  /**
   * Resolver
   *
   * @param route
   * @param state
   */
  resolve(
    route: ActivatedRouteSnapshot,
    state: RouterStateSnapshot
  ): Observable<PartnerBranch | undefined> {
    return this._partnerBranchService
      .getPartnerBranchById(route.paramMap.get('id'))
      .pipe(
        // Error here means the requested product is not available
        catchError((error) => {
          // Log the error
          console.error(error);

          // Get the parent url
          const parentUrl = state.url.split('/').slice(0, -1).join('/');

          // Navigate to there
          this._router.navigateByUrl(parentUrl);

          // Throw an error
          return throwError(error);
        })
      );
  }
}

@Injectable({
  providedIn: 'root',
})
export class PartnerBranchsResolver implements Resolve<any> {
  /**
   * Constructor
   */
  constructor(private _partnerBranchService: PartnerBranchService) {}

  // -----------------------------------------------------------------------------------------------------
  // @ Public methods
  // -----------------------------------------------------------------------------------------------------

  /**
   * Resolver
   *
   * @param route
   * @param state
   */
  resolve(
    route: ActivatedRouteSnapshot,
    state: RouterStateSnapshot
  ): Observable<{
    pagination: PartnerBranchPagination;
    partnerBranchs: PartnerBranch[];
  }> {
    return this._partnerBranchService.getPartnerBranchs();
  }
}
