import { Injectable } from '@angular/core';
import {
  ActivatedRouteSnapshot,
  Resolve,
  Router,
  RouterStateSnapshot,
} from '@angular/router';
import { catchError, Observable, throwError } from 'rxjs';

import { PartnerStore } from '../models/partner-store';
import { PartnerStorePagination } from '../models/partner-store-pagination';
import { PartnerStoreService } from '../services/partner-store.service';

@Injectable({
  providedIn: 'root',
})
export class PartnerStoreResolver implements Resolve<any> {
  /**
   * Constructor
   */
  constructor(
    private _partnerStoreService: PartnerStoreService,
    private _router: Router
  ) {}

  // -----------------------------------------------------------------------------------------------------
  // @ Public methods
  // -----------------------------------------------------------------------------------------------------

  /**
   * Resolver
   *
   * @param route
   * @param state
   */
  resolve(
    route: ActivatedRouteSnapshot,
    state: RouterStateSnapshot
  ): Observable<PartnerStore | undefined> {
    return this._partnerStoreService
      .getPartnerStoreById(route.paramMap.get('id'))
      .pipe(
        // Error here means the requested product is not available
        catchError((error) => {
          // Log the error
          console.error(error);

          // Get the parent url
          const parentUrl = state.url.split('/').slice(0, -1).join('/');

          // Navigate to there
          this._router.navigateByUrl(parentUrl);

          // Throw an error
          return throwError(error);
        })
      );
  }
}

@Injectable({
  providedIn: 'root',
})
export class PartnerStoresResolver implements Resolve<any> {
  /**
   * Constructor
   */
  constructor(private _partnerStoreService: PartnerStoreService) {}

  // -----------------------------------------------------------------------------------------------------
  // @ Public methods
  // -----------------------------------------------------------------------------------------------------

  /**
   * Resolver
   *
   * @param route
   * @param state
   */
  resolve(
    route: ActivatedRouteSnapshot,
    state: RouterStateSnapshot
  ): Observable<{
    pagination: PartnerStorePagination;
    partnerStores: PartnerStore[];
  }> {
    return this._partnerStoreService.getPartnerStores();
  }
}
